//	<><><><><><><><><><><><><>  Plot3DF.h  <><><><><><><><><><><><><><> 
//
// -----------------------------------------------------------------
// (C) Copyright Clark Dailey, Inc. 2006
//
// Plot3DF is a C++ class used to
// plot a 3D view of a function defined as f(x,y,z)=0
//
// Limitations:
//   f(x,y,z) must produce a sign change in order to be plottable
//
// -----------------------------------------------------------------

#ifndef	_PLOT3DF_H_	 	// avoid repeated including
#define	_PLOT3DF_H_	1	// has been included

#include "plot3d.h"


class Plot3DF : public Plot3D
{
	friend class CFormF;
public:
	// Clipping modes
	enum ClipMode
	{
	     CLIP_NONE=0,
	     CLIP_USER=1,
		 CLIP_CUBE=2,
		 CLIP_SPHERE=3,
	};

	Plot3DF();
	virtual ~Plot3DF();

	// load / save
	int ReadFile(void);
	int WriteFile(void);

	// plotting
	int Plot(CWnd* pWnd);
	void Reset(void);
	CString GetSurfName() { return(m_sSurfaceName); }

protected:
	// returns: 0=no, 1=point is visible
	int IsVisible(double xp,double yp,double zp);

	// clipping function; return: 0=clipped, 1=point is within drawable region
	int Within(double x,double y,double z);

	// surface function
	double Func(double x,double y,double z) { return( m_pFuncPlotCalc->Calc(x,y,z) ); }
	long  Color(double x,double y,double z);

	int  InitPlot(void);
	void DefInputs(void);
	int  InitEquations(void);
	void CalcDeltas(void);
	int  AllocateMaskArrays(void);
	void DeleteMaskArrays(void);
	void PlotMask(unsigned iy);
	int  DrawLines(void);
	int  PlotColorPixels(void);
	long CalcPixelColor(int xv,int yv);

protected:
	// ---------------
	// user input data
	// ---------------

	// surface definition
	CString m_sSurfaceName;
	CString	m_sC1;
	CString	m_sC2;
	CString	m_sC3;
	CString	m_sC4;
	CString	m_sC5;
	CString	m_sC6;
	CString	m_sC7;
	CString	m_sC8;
 	CString	m_sFuncEquation;
 	CString	m_sREquation;
 	CString	m_sGEquation;
 	CString	m_sBEquation;

	// number of constant zp planes
	unsigned m_nzplanes;

	// flags
	BOOL m_draw_lines;	   	// 0=no, 1=draw lines

	// clipping
	int		m_clip_mode;		// the clipping mode
	int		m_is_clip_inverted;	// 0=no, 1=invert clipped region (inside or outside)

	// spherical range values
	int		m_use_sphere_range;	// 0=use cube range, else use spherical range
	double	m_sphere_radius;	// radius of sphere
	double	m_sphere_xcenter;	// x value of center
	double	m_sphere_ycenter;	// y value of center
	double	m_sphere_zcenter;	// z value of center

	// -------------
	// internal data
	// -------------

	PlotCalc* m_pFuncPlotCalc; 	// Equation Calculator
	PlotCalc* m_pRPlotCalc;		// red    equation calculator
	PlotCalc* m_pGPlotCalc;		// green  equation calculator
	PlotCalc* m_pBPlotCalc;		// blue   equation calculator

	// min/max projection values
	double m_xpmin,  m_xpmax;
	double m_ypmin,  m_ypmax;
	double m_zpmin,  m_zpmax;

	// values used during plotting
	double m_dxp, m_dyp, m_dzp;

	// coordinate dimensions
	unsigned m_nXps, m_nYps, m_nZps;

	// masking point structure
	typedef struct mask_point_s
	{
		int within;	// 0=no, 1=within drawable area
		int visible;
		long color;
		double xp,yp,zp;
		double x, y, z;
		double f;	// function value
		int   sf;	// 0=(f<0), 0xffff=(f>=0)
	} MASK_POINT;

	// hidden line removal masking arrays
	MASK_POINT* m_pMask1;
	MASK_POINT* m_pMask2;
	unsigned m_mask_size;	// sizeof m_pMask1 and m_pMask2
};


#endif	/* if _PLOT3DF_H_ included */

/*	<><><><><><><><><><><><><>  Plot3DF.h  <><><><><><><><><><><><><><> */
